<?php

/*
 * Plugin Name: Jetpack Post Views
 * Plugin URI: http://blog.sklambert.com/jetpack-post-views/
 * Description: Adds a widget that displays your most popular posts using Jetpack stats.
 * Author: Steven Lambert
 * Version: 1.0.1
 * Author URI: http://sklambert.com
 * License: GPL2+
 */

// Set global Jetpack API URI
global $jetpack_post_view_apiUrlBeforeKey;
global $jetpack_post_view_apiUrlAfterKey;
$jetpack_post_view_apiUrlBeforeKey = 'http://stats.wordpress.com/csv.php';
$jetpack_post_view_apiUrlAfterKey = 'blog_uri='.get_bloginfo( 'wpurl' ).'&table=postviews&days=-1&limit=-1&summarize&format=json';

// Define plugin version
if ( !defined(' JETPACK_POST_VIEWS_VERSION_KEY' ) ) {
	define( 'JETPACK_POST_VIEWS_VERSION_KEY', 'jetpack-post-views_version' );
}
if ( !defined( 'JETPACK_POST_VIEWS_VERSION_NUM' ) ) {
	define( 'JETPACK_POST_VIEWS_VERSION_NUM', '1.0.1' );
}

/**
 * Jetpack Post Views
 * 
 * Queries the Jetpack API and adds a custom post_meta 'jetpack-post-views' that holds the total views of a 
 * post as listed on Jetpack. Adds a widget to display the top posts for a site. 
 * NOTE: Widget must have a correct Wordpress API Key before the post_meta data is added.
 */
class Jetpack_Post_Views extends WP_Widget {
	
	/* CONSTRUCTOR */
	public function Jetpack_Post_Views() {
		// Controls and options
		$widget_ops = array( 'classname' => 'jetpack-post-views', 'description' => __('Your site\'s most popular posts using Jetpack stats', 'jetpack-post-views') );  
        $control_ops = array( 'id_base' => 'jetpack-post-views-widget' );  
        $this->WP_Widget( 'jetpack-post-views-widget', __('Jetpack Post Views Widget', 'jetpack-post-views'), $widget_ops, $control_ops );
		
		add_option( 'jetpack_post_views_wp_api_key', '' ); // Wordpress API Key 
	}

	/* WIDGET OUTPUT */
	public function widget( $args, $instance ) {
		global $post;
		extract ( $args );

		$title = apply_filters('widget_title', $instance['title'] );

		// Grab the top posts and display them
		$args = array( 'numberposts' => $instance['num_posts'], 'orderby' => 'meta_value_num', 'order' => 'DESC', 'meta_key' => 'jetpack-post-views');
		$posts = get_posts( $args );

		echo $before_widget;

		if ( $title )
			echo $before_title . $title . $after_title;

		echo "<ul>";
		foreach( $posts as $post) { ?>
			<li><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a>
				<?php if ( $instance['show_views'] ) { 
					echo " - ".number_format_i18n( get_post_meta( $post->ID, 'jetpack-post-views', true ) )." views";
				} ?>
			</li>
		<?php
		}
		echo "</ul>";

		echo $after_widget;
	}

	/* UPDATE WIDGET OPTIONS */
	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		// Check nonce
		check_admin_referer('jetpack-post-views-widget-form-submition');

		$instance['title'] = strip_tags( $new_instance['title'] );
		$instance['api_key'] = strip_tags( $new_instance['api_key'] );
		$instance['show_views'] = strip_tags( $new_instance['show_views'] );
		$num_posts = intval( strip_tags( $new_instance['num_posts'] ) );
		
		// Set default number of posts to display if invalid option
		if ( $num_posts > 0 ) {
			$instance['num_posts'] = $num_posts;
		}
		else {
			$instance['num_posts'] = 5;
		}

		// Only invalidate an API Key on change
		if ( $instance['api_key'] != $old_instance['api_key'] ) {
			$instance['valid_key'] = false;
		}

		// Validate all API Keys
		if ( $instance['valid_key'] == false ) {
			global $jetpack_post_view_apiUrlBeforeKey;
			global $jetpack_post_view_apiUrlAfterKey;

			// Test the API Key
			$url = $jetpack_post_view_apiUrlBeforeKey.'?api_key='.$instance['api_key'].'&'.$jetpack_post_view_apiUrlAfterKey;
			$json = file_get_contents( $url );
			$data = json_decode( $json, true );

			if ( $data[0]['postviews'] ) { // We got data back, so the API Key is correct
				$this->get_stats( $data[0]['postviews'] ); // Retrieve all post stats and add the post meta

				// Validate the key
				$instance['valid_key'] = true;
				$instance['error'] = '';
				update_option( 'jetpack_post_views_wp_api_key', $instance['api_key'] );
			}
			else { // Incorrect API Key
				$instance['valid_key'] = false;

				if ( strlen($instance['api_key']) != 12 ) {
					$instance['error'] = __('Invalid API Key, must be 12 characters of 0123456789abcdef. Please visit https://apikey.wordpress.com/ to get your WP API Key.', 'jetpack-post-views');
				}
				else {
					$instance['error'] = __('Invalid API Key, does not belong to an administrator of blog. Please visit https://apikey.wordpress.com/ to get your WP API Key.', 'jetpack-post-views');
				}
			}
		}		

		return $instance;
	}

	/* DISPLAY WIDGET OPTIONS */
	public function form( $instance ) {

		// Default widget settings
		$defaults = array( 'title' => __('Most Popular Posts', 'jetpack-post-views'), 'api_key' => '', 'valid_key' => false, 'error' => '', 'num_posts' => 5, 'show_views' => false);
		$instance = wp_parse_args( (array) $instance, $defaults );

		// Set nonce
		if ( function_exists('wp_nonce_field') ) {
			wp_nonce_field('jetpack-post-views-widget-form-submition');
			echo "\n<!-- end of wp_nonce_field -->\n";
		}
	?>
		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'jetpack-post-views'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" type="text" />
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'api_key' ); ?>"><?php _e('WordPress API Key:', 'jetpack-post-views'); ?></label>
			<input class="widefat" id="<?php echo $this->get_field_id( 'api_key' ); ?>" name="<?php echo $this->get_field_name( 'api_key' ); ?>" value="<?php if ( $instance['api_key'] ) { echo $instance['api_key']; } ?>" placeholder="<?php _e("https://apikey.wordpress.com/", 'jetpack-post-views'); ?>" type="text" />
			<?php if ( $instance['error'] ) { ?>
			<div class="error"><?php echo $instance['error']; ?></div>
			<?php }	?>
		</p>

		<p>
			<label for="<?php echo $this->get_field_id( 'num_posts' ); ?>"><?php _e('Number of posts to show:', 'jetpack-post-views'); ?></label>
			<input id="<?php echo $this->get_field_id( 'num_posts' ); ?>" name="<?php echo $this->get_field_name( 'num_posts' ); ?>" value="<?php echo $instance['num_posts']; ?>" type="text" size="3" />
		</p>
		
		<p>
			<input class="checkbox" type="checkbox" <?php checked( $instance['show_views'], 'on' ); ?> id="<?php echo $this->get_field_id( 'show_views' ); ?>" name="<?php echo $this->get_field_name( 'show_views' ); ?>" /> 
			<label for="<?php echo $this->get_field_id( 'show_views' ); ?>"><?php _e('Display number of views?', 'jetpack-post-views'); ?></label>
		</p>

	<?php
	}

	/* 
	* GET STATS
	* Quries the Jetpack API for all posts and adds or updates the 'jetpack-post-views' post meta 
	*/
	private function get_stats( $postviews ) {
		if ( get_option( 'jetpack_post_views_stats_has_run' ) == "yes" ) { // Only run this function once
			return;
		}

		global $post;
		
		// Create a temporary array and save all data
		foreach ( $postviews as $postinfo ) {
			$postdata[ $postinfo['post_id'] ] = strip_tags( $postinfo['views'] );
		}

		// Grab all posts and update them
		$args = array( 'numberposts' => -1, 'post_type' => 'post', 'post_status' => 'publish');
		$allposts = get_posts( $args );
		foreach( $allposts as $post) {
			$views = intval( $postdata[ $post->ID ] );

			add_post_meta( $post->ID, 'jetpack-post-views', $views, true ) or update_post_meta( $post->ID, 'jetpack-post-views', $views );
		}

		update_option( 'jetpack_post_views_stats_has_run', 'yes' );
	}
}

/* REGISTER WIDGET */
function jetpack_post_views_register_widget() {  
    register_widget( 'Jetpack_Post_Views' );
}

/* 
* SCHEDULED UPDATE
* Update all posts 'jetpack-post-views' post meta
*/
function jetpack_post_views_scheduled_update() {
	$apikey = get_option( 'jetpack_post_views_wp_api_key', -1 );

	if ( $apikey != -1 ) { // Ensure we have an API Key set
		global $post;
		global $jetpack_post_view_apiUrlBeforeKey;
		global $jetpack_post_view_apiUrlAfterKey;

		$url = $jetpack_post_view_apiUrlBeforeKey.'?api_key='.$apikey.'&'.$jetpack_post_view_apiUrlAfterKey;

		// Get stats data
		$json = file_get_contents( $url );
		$data = json_decode( $json, true );
		$postviews = $data[0]['postviews'];

		// Create a temporary array and save all data
		foreach ( $postviews as $postinfo ) {
			$postdata[ $postinfo['post_id'] ] = strip_tags( $postinfo['views'] );
		}

		// Grab all posts and update them
		$args = array( 'numberposts' => -1, 'post_type' => 'post', 'post_status' => 'publish');
		$allposts = get_posts( $args );
		foreach( $allposts as $post) {
			$views = intval( $postdata[ $post->ID ] );

			add_post_meta( $post->ID, 'jetpack-post-views', $views, true ) or update_post_meta( $post->ID, 'jetpack-post-views', $views );
		}
	}
}

/* SET SCHEDULED EVENT */
function jetpack_post_views_on_activation() {
	wp_schedule_event( time() + 3600, 'hourly', 'jetpack_post_views_scheduled_update' );
}

/* UNSET SCHEDULED EVENT */
function jetpack_post_views_on_deactivation() {
	wp_clear_scheduled_hook( 'jetpack_post_views_scheduled_update' );
}

// Plugin options, actions, and hooks
add_option(JETPACK_POST_VIEWS_VERSION_KEY, JETPACK_POST_VIEWS_VERSION_NUM);

add_action( 'widgets_init', 'jetpack_post_views_register_widget' );
add_action( 'jetpack_post_views_scheduled_update', 'jetpack_post_views_scheduled_update' );

register_activation_hook( __FILE__, 'jetpack_post_views_on_activation' );
register_deactivation_hook( __FILE__, 'jetpack_post_views_on_deactivation' );